// MDR
// version 1.0
// Sept 10, 2014
// Author: Vi Q. Ha


#ifndef VAMDE_R
#define VAMDE_R

#include "stdio.h"
#include "TYPES.h"
#include "Parameters.h"
#include <boost/mpi.hpp>

namespace mdr {

class MDR {
public:


	
	
	mdr::Parameters * parameters;// Edited By Vi on 2014/12/14
	//~ 
	class Props * properties;
	class Data * data;
	class Thermo * thermostat;
	class Neighbor * neighbor;

 
	CONTEXT_TYPE * context;
	NETWORK_TYPE * agentNetwork;// Edited By Vi on 2014/09/10
	SPACE_TYPE * continuousSpace;// Edited By Vi on 2014/08/13
	repast::Properties * props;
	boost::mpi::communicator * comm;
	
	std::map<int, mdr::AgentTypeCard > type_list;
	MDR(CONTEXT_TYPE * _context,SPACE_TYPE * _continuousSpace,NETWORK_TYPE * _agentNetwork,boost::mpi::communicator * _comm,repast::Properties * _props);
	~MDR();

	int world_size(){int world_size;MPI_Comm_size(*comm, &world_size);return world_size;};
	int world_rank(){int world_rank;MPI_Comm_rank(*comm, &world_rank);return world_rank;};
	int rank(){return world_rank();};
	int nRank(){return world_size();};
	int nProc(){return world_size();};
	int tick(){return repast::RepastProcess::instance()->getScheduleRunner().currentTick();};
	bool boss(){return ( rank() == 0 ) ; }
	// Vi's edit 
	
	/// Basic MD variables
	double rcut; 
	int NDIM ; // 2014.05.07 Vi's edit // These will be changed
	dinner::Line<double> worldSize ; // 2014.05.07 Vi's edit
	
	/// Neighbor List Stuff
	int numNetworkConnections;
	int nebrNow;
	double dispHi;
	
	// 2014.06.04 Vi's edit 
	/// Yihao's 3D:
	int PrNx, PrNy, PrNz, PrNtotal;
	int buffer;
	
	/// Initiates Simulations, (assigning initial parameters)
	
	dinner::Vector<double> getLoc(AGENT_TYPE * it){
		
		std::vector<double> locVec;
		
		continuousSpace->getLocation(it, locVec);
		
		repast::Point<double> Location(locVec);
		
		return dinner::Vector<double>(Location);
		
	}
	
	
	dinner::Vector<double> getLoc(repast::AgentId& id){
		
		AGENT_TYPE * agent = context->getAgent(id);
		return getLoc(agent);
		
	}
	std::ofstream vmdOut;
	std::ofstream vmdRank;
	
	
	dinner::Tensor<long double> H;
	dinner::Tensor<long double> H_o;
	
	void GetPositions();
	void MoveParticle();
	
	void ApplyStrain();
	void TransformBox();
	void UntransformBox();
	void UnwrapNonlocal();
	void Transform_test();
	
	void zeroAcc();
	void LeapFrogStep(int PART);
	
	bool agentIsLocal(AGENT_TYPE * agt)
	{
		
		return (agt->getId().currentRank() == world_rank() );

		
	}
	
	void print_pos(int flag)
	{
		this->vmdRank << "Tick: " << tick() << ", Flag " << flag << std::endl;
		repast::GridDimensions home = continuousSpace->bounds();
		double xMin = home.origin().getX();
		double xMax = xMin + home.extents().getX();
		double yMin = home.origin().getY();
		double yMax = yMin + home.extents().getY();
		double zMin = home.origin().getZ();
		double zMax = zMin + home.extents().getZ();
		this->vmdRank << "{" << xMin << " , "  << yMin << " , "  << zMin << "};{" << xMax << " , "  << yMax << " , "  << zMax << "}" << std::endl;
		
		repast::AgentId id(0, 0, 0);
		AGENT_TYPE * agent1 = context->getAgent(id);
		this->vmdRank << agent1->position << std::endl;

	}
	void print_all(int flag)
	{
		this->vmdRank << "Tick: " << tick() <<  ", Flag: " << flag << std::endl;
		repast::GridDimensions home = continuousSpace->bounds();
		double xMin = home.origin().getX();
		double xMax = xMin + home.extents().getX();
		double yMin = home.origin().getY();
		double yMax = yMin + home.extents().getY();
		double zMin = home.origin().getZ();
		double zMax = zMin + home.extents().getZ();
		this->vmdRank << "{" << xMin << " , "  << yMin << " , "  << zMin << "};{" << xMax << " , "  << yMax << " , "  << zMax << "}" << std::endl;
		
		std::set<AGENT_TYPE*> agents;
		this->context->selectAgents( agents, false);
		std::set<AGENT_TYPE*>::iterator it = agents.begin();
		while(it != agents.end())
		{
			if (!(agentIsLocal(*it)) ) this->vmdRank << this-> agentIsLocal(*it) << "  " << (*it)->getId()  << "  " << (*it)->position<< "  " << (*it)->orientation << std::endl;

			it++;
		}

	}
	
	
};

}

#endif
