// MDR
// version 1.0
// December 14, 2014
// Author: Vi Q. Ha


#ifndef VMD_PAIR_POT
#define VMD_PAIR_POT


#include "TYPES.h"
#include "MDR.h"
#include "props.h"

namespace mdr {
	
	

class PairPotential
{
    private:
	protected:
		std::set< std::vector<int> > specificities;
    public:

		mdr::MDR * vmd;
		PairPotential(mdr::MDR * _vmd): vmd(_vmd){}
		~PairPotential(){}
		
		void addtype(int type0, int type1);
		void removetype(int type0, int type1);
		bool check_specificity(int type0, int type1);
		
		void print_specificity();
		void print_specificity_size(){ std::cout << "specified pair size: " << specificities.size() << std::endl;}

};

class LJPotential : public PairPotential
{
    private:
		bool isRepulsive;
		
    public:
		LJPotential(mdr::MDR * _vmd):PairPotential(_vmd){isRepulsive = false;}
		~LJPotential(){}
		PAIR_LIST_TYPE apply(PAIR_LIST_TYPE * NeighborList );
		void set_attractive(){isRepulsive = false;}
		void set_repulsive(){isRepulsive = true;}

};

class MembranePotential : public PairPotential
{
    private:
    public:
		MembranePotential(mdr::MDR * _vmd):PairPotential(_vmd){}
		~MembranePotential(){}
		PAIR_LIST_TYPE apply(PAIR_LIST_TYPE * NeighborList );

};

class SpecificPotential
{
    private:
		//~ bool isRepulsive;
		int type0, type1;
		
    public:
		mdr::MDR * vmd;
		SpecificPotential(mdr::MDR * _vmd): vmd(_vmd){}
		~SpecificPotential(){}
		bool check_specificity(int _type0, int _type1);
		void addtype(int _type0, int _type1){type0 = _type0; type1 = _type1;}
		//~ PAIR_LIST_TYPE apply(PAIR_LIST_TYPE * NeighborList );
		//~ void set_attractive(){isRepulsive = false;}
		//~ void set_repulsive(){isRepulsive = true;}
		void unbinding_broken();
		void bind_attached(PAIR_LIST_TYPE * NeighborList );
		void apply(PAIR_LIST_TYPE * NeighborList );

};
} // end namespace
#endif
