// MDR
// version 1.0
// December 17, 2014
// Author: Vi Q. Ha


#ifndef VMD_SIM_INIT
#define VMD_SIM_INIT
#include "initialize_random.h"
#include <boost/mpi.hpp>


#include "TYPES.h"
#include "MDR.h"
#include <stdlib.h>     /* srand, rand */
#include <math.h>
namespace mdr {
	
	

class SimulationInitiator
{
    private:
		//~ std::set< std::pair<int , double> > type_distribution;
		std::map<int, double> type_distribution;
    	//~ int nMolMe;
		bool random_position, random_velocity, ramdom_acceleration;
		double const_v, min_v, max_v;
		double const_a, min_a, max_a;
		int nx , ny, nz;
		int nAll;
		int rank ;
		double drx, dry, drz, dvx, dvy, dvz, dax, day, daz;
		int default_type;
		dinner::Vector<long double> default_orientation;
	protected:
		mdr::MDR * vmd;
    public:
		
		SimulationInitiator(mdr::MDR * _vmd);
		~SimulationInitiator(){}
		void completeParticle(dinner::Vector<long double> possition0);
		void completeParticle(dinner::Vector<long double> possition0, dinner::Vector<long double> orientation0);

		void reset();
		
		void set_default_type(int type0){ default_type = type0;}
		void add_type_distribution(int _type, double _distribution){type_distribution[_type] = _distribution;}
		int get_type();
		
		void set_ordered_position_distribution(int _nx ,int _ny, int _nz);
		void set_random_position_distribution(int _nAll);
		void shift_position(double _drx, double _dry, double _drz);
		
		void set_constant_velocity(double _const_v);
		void set_random_velocity(double _min_v, double  _max_v);
		void shift_velocity(double _dvx, double _dvy, double _dvz);
		
		void set_constant_acceleration(double _const_a);
		void set_random_acceleration(double _min_a, double  _max_a);
		void shift_acceleration(double _dax, double _day, double _daz);

		void generate();
		void generate_sphere(double radius, double n);
};


} // end namespace
#endif
